/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include "ImageClientConverter.h"

ImageClientConverter::ImageClientConverter()
  : m_client_rect(0,0,0,0), m_image_size(0,0), m_window(0)
{
}

ImageClientConverter::~ImageClientConverter()
{
}

CRect ImageClientConverter::GetClientRect(int window) const
{
  CRect r = m_client_rect;
  CPoint offset( 0, (window - m_window) * m_client_rect.Height() );
  r.OffsetRect( offset );
  return r;
}

CRect ImageClientConverter::GetExtendedClientRect(int window1, int window2) const
{
  CRect r1 = GetClientRect(window1);
  CRect r2 = GetClientRect(window2);
  CRect r;
  r.UnionRect(r1, r2);
  return r;
}

RPoint ImageClientConverter::ClientToImage(const CPoint& point) const
{
  // avoid divide by zero
  // (needed b/c onmouse move calls this function before the class is initialized)
  if ( m_client_rect.Width() == 0 
    || m_client_rect.Height() == 0 
    || m_image_size.cy == 0 )  
  { 
    return RPoint(0, 0, 0);
  } 
  else
  {
    double scale_x = ((double) m_image_size.cx) / m_client_rect.Width();
    double scale_y = ((double) m_image_size.cy) / m_client_rect.Height();
    double ix = (point.x - m_client_rect.left) * scale_x;
    double iy = (point.y - m_client_rect.top ) * scale_y;
    int iix = (int) (ix + 0.5);  // round to nearest integer
    int iiy = (int) (iy + 0.5);  // round to nearest integer
    int w = iiy / m_image_size.cy;

    if (iiy < 0)  w--;
    iiy -= (m_image_size.cy * w);

    w += m_window;
    return RPoint( iix, iiy, w );
  }
}

RPoint ImageClientConverter::ClientToImage(const CPoint& point, int window) const
{
  RPoint rpt = ClientToImage(point);
  rpt.y += m_image_size.cy * (rpt.window - window);
  rpt.window = window;
  return rpt;
}

CPoint ImageClientConverter::ImageToClient(const CPoint& point, int window) const
{
  bool okay = (   m_client_rect.Width() > 0 && m_client_rect.Height() > 0
               && m_image_size.cx > 0 && m_image_size.cy > 0);
  assert(okay);
  if (!okay)  return CPoint(0, 0); // needed b/c onmouse move calls this function before the class is initialized

  int left = m_client_rect.left;
  int top = m_client_rect.top + (window - m_window) * m_client_rect.Height();
  double scale_x = ((double) m_image_size.cx) / m_client_rect.Width();
  double scale_y = ((double) m_image_size.cy) / m_client_rect.Height();
  if(scale_x == 0) scale_x = 1;
  if(scale_y == 0) scale_y = 1;
  double ix = left + (point.x / scale_x);
  double iy = top  + (point.y / scale_y);
  int iix = (int) (ix + 0.5); //round to nearest integer
  int iiy = (int) (iy + 0.5); //round to nearest integer
  return CPoint( iix, iiy );
}

RPoint ImageClientConverter::ChangeWindow(const RPoint& point, int window) const
{
  RPoint rpt = point;
  rpt.y += m_image_size.cy * (rpt.window - window);
  rpt.window = window;
  return rpt;
}


