#include "stdafx.h"
#include "Rootfly.h"
#include "RootflyDlg.h"

double CalculateDiameterUsingScalingFactor(const RDiameter& diam, RPoint pt, CRootflyDlg* dlg)
{
  double factor = dlg->m_rfy_data->GetSettings().GetDiameterScalingFactor();
  factor = 3;  // !! should not override value! (fixed for now just to be safe -- b/c we didn't have time to test)
  
  int initial_x = diam.r_point.x;
  int initial_y = diam.r_point.y;
  int delta_x = floor(fabs(initial_x - pt.x) / factor);
  int delta_y = floor(fabs(initial_y - pt.y) / factor);
  
  pt.x = (initial_x > pt.x) ? initial_x - delta_x : initial_x + delta_x;
  pt.y = (initial_y > pt.y) ? initial_y - delta_y : initial_y + delta_y;
  
  double new_radius = sqrt( pow(( (float)pt.x - initial_x),2) + pow(( (float)pt.y - initial_y),2) );
  return new_radius;
}

// !!! These two functions
//       ResizeDiameterUsingScalingFactor and
//       ResizeCurrentDiameterUsingScalingFactor
//     should probably be refactored.
void ResizeDiameterUsingScalingFactor(CPoint point2, CRootflyDlg* dlg)
{
  SetCursor(dlg->m_cur_sizens);    
  const Tube *tube = dlg->m_rfy_data->GetExperiment().GetTubeByName(dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(dlg->m_rfy_data->GetApplicationData().GetCurrentSession());
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  RPoint pt = dlg->m_converter.ClientToImage( CPoint(point2.x,point2.y),root->GetPrimaryWindow());

  double new_radius = CalculateDiameterUsingScalingFactor(*diam, pt, dlg);
  dlg->m_controller.C_ResizeDiameter(0, dlg->m_selected_root_id, dlg->m_point_session, new_radius);      
}

void ResizeCurrentDiameterUsingScalingFactor(CPoint point, CRootflyDlg* dlg)
{
  vector<int> window_list;
  dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
  RPoint pt = dlg->m_converter.ClientToImage(point);
  pt = dlg->m_converter.ChangeWindow(pt ,window_list[0]);
  RDiameter *diam = dlg->m_current_root_being_drawn.GetDiameter(0);

  double new_radius = CalculateDiameterUsingScalingFactor(*diam, pt, dlg);
  diam->radius = new_radius;
}

////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_STARTINGMOVEALL

void ModeHandlerMODE_STARTINGMOVEALL::OnMouseMove(UINT nFlags, CPoint point2)
{
}

void ModeHandlerMODE_STARTINGMOVEALL::OnLButtonDown(UINT nFlags, CPoint point)
{
  m_dlg->m_previous_click_cpoint_for_undo = point;
  RPoint pt = m_dlg->m_converter.ClientToImage( point );
  //preserve the starting point of the move all
  m_dlg->m_controller.C_SetShowRootID(false); //fix clipping error
  m_dlg->m_last_mouse_move_point = CPoint(pt.x, pt.y);
  m_dlg->SetMode(MODE_MOVINGALLROOTS);
}

void ModeHandlerMODE_STARTINGMOVEALL::OnLButtonUp(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_STARTINGMOVEALL::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_STARTINGMOVEALL::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_STARTINGMOVEALL::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_STARTINGMOVEALL::GetStatusText()
{
  return "Left click to begin moving all roots";
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_MOVINGSINGLEROOT

void ModeHandlerMODE_MOVINGSINGLEROOT::OnMouseMove(UINT nFlags, CPoint cpoint)
{
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName() );
  assert(tube); if (!tube) return;
  const Root *tempr = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(tempr);  if (!tempr)  return;
  const RootInstance *inst = tempr->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  RPoint point = m_dlg->m_converter.ClientToImage(CPoint(cpoint.x,cpoint.y),tempr->GetPrimaryWindow());
  
  CPoint last = m_dlg->m_last_mouse_move_point;	
  int session = m_dlg->m_point_session;
  m_dlg->m_controller.C_MoveRootSess( m_dlg->m_selected_root_id, m_dlg->m_point_session, point.x - last.x, point.y - last.y);   
  //update the new old point
  m_dlg->m_last_mouse_move_point = CPoint( point.x, point.y );
  m_dlg->ReDraw(1,0);
}

CString ModeHandlerMODE_MOVINGSINGLEROOT::GetStatusText()
{
  return "Moving root, left click to finish";
}

void ModeHandlerMODE_MOVINGSINGLEROOT::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_MOVINGSINGLEROOT::OnLButtonDown(UINT nFlags, CPoint cpoint)
{
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName() );
  assert(tube); if (!tube) return;
  const Root *tempr = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(tempr);  if (!tempr)  return;
  const RootInstance *inst = tempr->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  RPoint point = m_dlg->m_converter.ClientToImage(CPoint(cpoint.x,cpoint.y),tempr->GetPrimaryWindow());
  
  RPoint undo_point = m_dlg->m_converter.ClientToImage( m_dlg->m_previous_click_cpoint_for_undo,tempr->GetPrimaryWindow());
  CString temp;
  CPoint last = m_dlg->m_last_mouse_move_point;
  int session = m_dlg->m_point_session;	
  m_dlg->m_controller.C_MoveRootSessU( m_dlg->m_selected_root_id, m_dlg->m_point_session, point.x - last.x, point.y - last.y, point.x - undo_point.x, point.y - undo_point.y );
  
  //finished moving root
  m_dlg->SetMode(MODE_ROOTSELECTED);
  if (m_dlg->m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  { //Marks the root to show the numerical ID of the root
    m_dlg->m_controller.C_SetShowRootID(true);
  }
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_MOVINGSINGLEROOT::OnLButtonUp(UINT nFlags, CPoint cpoint)
{
}

void ModeHandlerMODE_MOVINGSINGLEROOT::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_MOVINGSINGLEROOT::OnRButtonUp(UINT nFlags, CPoint point)
{
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_MOVINGALLROOTS

void ModeHandlerMODE_MOVINGALLROOTS::OnMouseMove(UINT nFlags, CPoint point2) 
{
  int point_session = m_dlg->m_point_session;
  int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint point = m_dlg->m_converter.ClientToImage( point2,window);
  
  CPoint last = m_dlg->m_last_mouse_move_point;
  
  m_dlg->m_controller.C_MoveRootsInWindow(window, point_session, point.x - last.x, point.y - last.y);    
  
  //update the new old point
  m_dlg->m_last_mouse_move_point = CPoint( point.x, point.y );
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_MOVINGALLROOTS::OnLButtonDown(UINT nFlags, CPoint point2) 
{
  int point_session = m_dlg->m_point_session;
  int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint point = m_dlg->m_converter.ClientToImage( point2,window);
  RPoint undo_point = m_dlg->m_converter.ClientToImage( m_dlg->m_previous_click_cpoint_for_undo,window);
  
  CPoint last = m_dlg->m_last_mouse_move_point;
  
  m_dlg->m_controller.C_MoveRootsInWindow_U(window, point_session, point.x - last.x, point.y - last.y, point.x - undo_point.x, point.y - undo_point.y);  
  
  //finished with move all
  m_dlg->SetMode(MODE_IMAGEDISPLAYED);
  if (m_dlg->m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  { //Marks the root to show the numerical ID of the root
    m_dlg->m_controller.C_SetShowRootID(true);
  }
  m_dlg->ReDraw(1,0);    
}

void ModeHandlerMODE_MOVINGALLROOTS::OnLButtonUp(UINT nFlags, CPoint point2) 
{
}

void ModeHandlerMODE_MOVINGALLROOTS::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_MOVINGALLROOTS::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_MOVINGALLROOTS::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_MOVINGALLROOTS::GetStatusText() 
{
  return "Moving roots, left click to finish";
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_NOFILELOADED

void ModeHandlerMODE_NOFILELOADED::OnMouseMove(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_NOFILELOADED::OnLButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_NOFILELOADED::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_NOFILELOADED::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_NOFILELOADED::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_NOFILELOADED::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_NOFILELOADED::GetStatusText() 
{ 
  return "To begin, choose File->New or File->Open"; 
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_DRAWINGNEWROOT

void ModeHandlerMODE_DRAWINGNEWROOT::OnMouseMove(UINT nFlags, CPoint point) 
{  
  const Tube* currTube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  vector<int> window_list;
  m_dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
  RPoint temp = m_dlg->m_converter.ClientToImage(point);
  if (!currTube->GetImageExists(temp.window,m_dlg->m_point_session))
    return;
  SetCursor(m_dlg->m_cur_pen);
  m_dlg->ReDraw( 1, 0 );
}

void ModeHandlerMODE_DRAWINGNEWROOT::OnLButtonDown(UINT nFlags, CPoint point) 
{
  const Tube* currTube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  vector<int> window_list;
  m_dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
  RPoint temp = m_dlg->m_converter.ClientToImage(point);
  if (!currTube->GetImageExists(temp.window,m_dlg->m_point_session))
    return;
  if(window_list.size() > 0)
    temp = m_dlg->m_converter.ChangeWindow(temp,window_list[0]);
  SetCursor(m_dlg->m_cur_pen);
  RPoint *rpt = m_dlg->m_current_root_being_drawn.CreatePoint();    
  assert(rpt); if( !rpt ) return;
  rpt->x = temp.x;
  rpt->y = temp.y;
  rpt->window = temp.window;  
  
  m_dlg->ReDraw(1, 0); 
}

void ModeHandlerMODE_DRAWINGNEWROOT::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_DRAWINGNEWROOT::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
  
  const Tube* currTube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  RDiameter *rdm;
  int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint conv_point = m_dlg->m_converter.ClientToImage( CPoint(point.x,point.y),window);
  RPoint temp = m_dlg->m_converter.ClientToImage( point );
  while ( m_dlg->m_current_root_being_drawn.GetAllPoints()->size() < 2 ) 
  {
    if (!currTube->GetImageExists(temp.window,m_dlg->m_point_session))
      return;
    SetCursor(m_dlg->m_cur_pen);
    RPoint *rpt = m_dlg->m_current_root_being_drawn.CreatePoint();    
    assert(rpt); if( !rpt ) return;
    rpt->x = temp.x;
    rpt->y = temp.y;
    rpt->window = temp.window;       
    m_dlg->ReDraw(1, 0);  
  }
  if (!currTube->GetImageExists(temp.window,m_dlg->m_point_session))
    return;
  rdm = m_dlg->m_current_root_being_drawn.CreateDiameter();
  rdm->r_point.x = conv_point.x;
  rdm->r_point.y = conv_point.y;
  rdm->radius = 8;
  rdm->r_point.window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  m_dlg->SetMode(MODE_LOCATINGDIAMETER); 
}

void ModeHandlerMODE_DRAWINGNEWROOT::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_DRAWINGNEWROOT::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_DRAWINGNEWROOT::GetStatusText() 
{ 
  return "Left click to add point, double click to end root"; 
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_LOCATINGDIAMETER

void ModeHandlerMODE_LOCATINGDIAMETER::OnMouseMove(UINT nFlags, CPoint point) 
{
  vector<int> window_list;
  m_dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
  RPoint pt = m_dlg->m_converter.ClientToImage(point);
  
  RDiameter *rdm = m_dlg->m_current_root_being_drawn.GetDiameter(0);
  RPoint temp = m_dlg->FindNearestPointOnRoot( pt, m_dlg->m_current_root_being_drawn);    
  temp = m_dlg->m_converter.ChangeWindow(temp,window_list[0]);
  rdm->r_point.x = temp.x;
  rdm->r_point.y = temp.y;
  rdm->r_point.window = temp.window; 
  rdm->radius = 8;    
  m_dlg->ReDraw(1, 0);   
}

void ModeHandlerMODE_LOCATINGDIAMETER::OnLButtonDown(UINT nFlags, CPoint point) 
{
  ResizeCurrentDiameterUsingScalingFactor(point, m_dlg);
//  { // same code as ModeHandlerMODE_SIZINGDIAMETER::OnMouseMove
//    vector<int> window_list;
//    m_dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
//    RPoint pt = m_dlg->m_converter.ClientToImage(point);
//    pt = m_dlg->m_converter.ChangeWindow(pt, window_list[0]);    
//    RDiameter *rdm = m_dlg->m_current_root_being_drawn.GetDiameter(0);
//    rdm->radius = sqrt( pow(( (float)pt.x - rdm->r_point.x),2) + pow(( (float)pt.y - rdm->r_point.y),2) ); 
//  }
  m_dlg->SetMode(MODE_SIZINGDIAMETER);       
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_LOCATINGDIAMETER::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_LOCATINGDIAMETER::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_LOCATINGDIAMETER::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_LOCATINGDIAMETER::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_LOCATINGDIAMETER::GetStatusText() 
{
  return "Left click to place diameter";	
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_REPOSITIONDIAMETER

void ModeHandlerMODE_REPOSITIONDIAMETER::OnMouseMove(UINT nFlags, CPoint point2) 
{
  RPoint point = m_dlg->m_converter.ClientToImage( point2 );
  
  SetCursor(m_dlg->m_cur_closedhand);
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  RPoint temp = m_dlg->FindNearestPointOnRoot(point, *inst);
  temp = m_dlg->m_converter.ChangeWindow(temp,root->GetPrimaryWindow());
  double dist = sqrt(pow(((float)temp.x - point.x),2) + pow(((float)temp.y - point.y),2));       
  if ( dist > 1.0 )  //keeps the diameter from walking around
  {
    m_dlg->m_controller.C_MoveDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, temp);   
  } 
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_REPOSITIONDIAMETER::OnLButtonDown(UINT nFlags, CPoint point2) 
{
  RPoint point = m_dlg->m_converter.ClientToImage( point2 );
  
  SetCursor(m_dlg->m_cur_closedhand);
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  RPoint temp = m_dlg->FindNearestPointOnRoot(point, *inst);
  temp = m_dlg->m_converter.ChangeWindow(temp,root->GetPrimaryWindow());
  double dist = sqrt(pow(((float)temp.x - point.x),2) + pow(((float)temp.y - point.y),2));       
  if ( dist > 1.0 )  //keeps the diameter from walking around
  {
    m_dlg->m_controller.C_MoveDiameter_U(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, m_dlg->m_previous_diam_rpoint_for_undo, temp);   
  } 
  
  m_dlg->SetMode(MODE_ROOTSELECTED);          
  m_dlg->ReDraw(1, 0);
  
}

void ModeHandlerMODE_REPOSITIONDIAMETER::OnLButtonUp(UINT nFlags, CPoint point2) 
{
}

void ModeHandlerMODE_REPOSITIONDIAMETER::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_REPOSITIONDIAMETER::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_REPOSITIONDIAMETER::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_REPOSITIONDIAMETER::GetStatusText()
{ 
  return "Moving diameter, left click to finish"; 
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_ROOTSELECTED

void ModeHandlerMODE_ROOTSELECTED::OnMouseMove(UINT nFlags, CPoint point_client)
{
  const int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint point = m_dlg->m_converter.ClientToImage( point_client, window );
  RDiameter nearest_point;
  nearest_point.radius = -1;
  
  m_dlg->FindNearestPoint( point.x, point.y, &nearest_point);   
  m_dlg->m_over_root = ( nearest_point.radius >= 0 );
  m_dlg->m_selected_index = m_dlg->GetSelectedIndex(CPoint( point.x, point.y ));    
  switch ( m_dlg->m_selected_index )
  {
  case m_dlg->RESIZING_DIAMETER_SELECTED:
    { // mouse at edge of diameter:  indicate that 'resizing diameter' is allowed
      SetCursor(m_dlg->m_cur_sizens);
      break;
    }
  case m_dlg->DIAMETER_SELECTED:
    { // mouse inside diameter:  indicate that 'move diameter' is allowed
      SetCursor(m_dlg->m_cur_openhand);
      break;
    }
  case m_dlg->NOTHING_SELECTED:
    {
      if ( m_dlg->m_over_root )
      { // mouse over polyline:  indicate that 'move root' is allowed
        SetCursor(m_dlg->m_cur_sizeall); 
      }
      break;
    }
  default:
    if ( m_dlg->m_selected_index >= 0 )
    { // mouse over point:  indicate that 'move point' is allowed
      SetCursor(m_dlg->m_cur_movepoint); 
    }
    break;
  }
  
  m_dlg->UpdateStatusText();
}

void ModeHandlerMODE_ROOTSELECTED::OnLButtonDown(UINT nFlags, CPoint point_client)
{
  const int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint point = m_dlg->m_converter.ClientToImage( point_client, window );
  //user clicked a root in different session, set session then return (perform no action but selects new root) -MTB
  if(m_dlg->m_selected_root_session != m_dlg->m_point_session)
  {
    m_dlg->m_selected_root_id = -1;
    m_dlg->SetMode(MODE_IMAGEDISPLAYED);
    return;
  }
  
  RDiameter nearest_point;
  m_dlg->FindNearestPoint( point.x, point.y, &nearest_point);      
  int selected_diameter = m_dlg->FindNearestDiameter(point.x, point.y);
  // Check to see if another root has been selected and change selected ID
  {
    bool clicked_point_of_selected_root = nearest_point.radius >= 0 && nearest_point.r_point.window == m_dlg->m_selected_root_id;
    bool clicked_diameter_of_selected_root = selected_diameter >= 0 && selected_diameter == m_dlg->m_selected_root_id;
    bool clicked_point_of_another_root = nearest_point.radius >= 0 && nearest_point.r_point.window != m_dlg->m_selected_root_id;
    bool clicked_diameter_of_another_root = selected_diameter >= 0 && selected_diameter != m_dlg->m_selected_root_id;
    if (!clicked_point_of_selected_root && !clicked_diameter_of_selected_root)
    {
      if (clicked_diameter_of_another_root) 
      {
        m_dlg->m_selected_root_id = selected_diameter;
        m_dlg->ReDraw(1,0);
        return;
      }
      else if (clicked_point_of_another_root) 
      {
        m_dlg->m_selected_root_id = nearest_point.r_point.window;  
        m_dlg->ReDraw(1,0);
        return;
      }
    }
  }
  // -1 means not over an interactive area of the root
  // -2 means over the diameter move area   (now moving the diameter)
  // -3 means over the diameter resize area (now resizing the diameter)   
  // all other indexes refer to points along the root (now moving a point)
  if (m_dlg->m_selected_index == m_dlg->RESIZING_DIAMETER_SELECTED) // -3
  { // mouse at edge of diameter:  indicate that 'resize diameter' is allowed
    SetCursor(m_dlg->m_cur_sizens);
    
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
    assert(root);  if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
    assert(inst);  if (!inst)  return;
    const RDiameter *diam = inst->GetDiameter(0);
    assert(diam);  if (!diam)  return;
    m_dlg->m_previous_radius_for_undo = diam->radius;
    
    m_dlg->SetMode(MODE_RESIZINGDIAMETER);
  }
  else if ( m_dlg->m_selected_index == m_dlg->DIAMETER_SELECTED )  // -2
  { // mouse inside diameter:  indicate that 'move diameter' is allowed
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
    assert(root);  if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
    assert(inst);  if (!inst)  return;
    const RDiameter *diam = inst->GetDiameter(0);
    assert(diam);  if (!diam)  return;
    m_dlg->m_previous_diam_rpoint_for_undo = diam->r_point;
    
    SetCursor(m_dlg->m_cur_closedhand);
    m_dlg->SetMode(MODE_REPOSITIONDIAMETER);
  }
  else if ( m_dlg->m_selected_index >= 0 )
  { // mouse over point:  indicate that 'move point' is allowed
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
    assert(root);  if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
    assert(inst);  if (!inst)  return;
    m_dlg->m_previous_root_for_undo = *inst;
    
    SetCursor(m_dlg->m_cur_movepoint);
    m_dlg->SetMode(MODE_MOVINGRPOINT); 
    m_dlg->UpdateDiameterPositionRatio();
  }
  else if (!m_dlg->m_over_root) 
  {
    // clicked off of root, it is no longer selected
    m_dlg->SetMode(MODE_IMAGEDISPLAYED);      
    m_dlg->m_selected_root_id = -1;
    m_dlg->m_selected_root_session = -1;
    m_dlg->m_selected_index = -1;       
    m_dlg->ReDraw(1, 0);
  }
  else
  {
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
    assert(root);  if (!root)  return;
    const int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
    
    m_dlg->m_previous_click_cpoint_for_undo = point_client;
    //preserve the starting point of the move 
    m_dlg->m_controller.C_SetShowRootID(false); //fix clipping error
    
    // Adjusting y coordinate by number of windows the root's primary windows differs from the current selected window.
    point.y += m_dlg->m_converter.GetImageSize().cy * (- root->GetPrimaryWindow() + window);
    m_dlg->m_last_mouse_move_point = CPoint(point.x, point.y);
    m_dlg->SetMode(MODE_MOVINGSINGLEROOT);
  }
}

void ModeHandlerMODE_ROOTSELECTED::OnLButtonUp(UINT nFlags, CPoint point_client)
{
}

void ModeHandlerMODE_ROOTSELECTED::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_ROOTSELECTED::OnRButtonDown(UINT nFlags, CPoint point) 
{
  int window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint rPoint = m_dlg->m_converter.ClientToImage( point, window );  
  RDiameter nearest_point;
  
  m_dlg->FindNearestPoint( rPoint.x, rPoint.y, &nearest_point);      
  int selected_diameter = m_dlg->FindNearestDiameter(rPoint.x, rPoint.y);
  
  m_dlg->m_selected_index = m_dlg->GetSelectedIndex(CPoint(rPoint.x, rPoint.y));
  
  // Check to see if another root has been selected and change selected ID
  // Note:  'point' here does not refer to an RPoint defining a vertex of the root's polyline;
  //        Rather it refers to a place along the polyline.  I.e., if the user clicks on the polyline
  //        but not near a vertex, the nearest_point.radius will still be small and 
  //        'clicked_point_of_selected_root' will be true. -- STB 7/2/09
  bool clicked_point_of_selected_root = nearest_point.radius >= 0 && nearest_point.r_point.window == m_dlg->m_selected_root_id;
  bool clicked_diameter_of_selected_root = selected_diameter >= 0 && selected_diameter == m_dlg->m_selected_root_id;
  bool clicked_point_of_another_root = nearest_point.radius >= 0 && nearest_point.r_point.window != m_dlg->m_selected_root_id;
  bool clicked_diameter_of_another_root = selected_diameter >= 0 && selected_diameter != m_dlg->m_selected_root_id;
  
  MenuSetup(point);
}

void ModeHandlerMODE_ROOTSELECTED::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_ROOTSELECTED::GetStatusText() 
{ 
  CString str;
  CString add_options = ", right click for additional options.";
  switch( m_dlg->m_selected_index )
  {
  case m_dlg->RESIZING_DIAMETER_SELECTED:   // -3
    str+= _T("Left click to resize this diameter") + add_options;
    break;
  case m_dlg->DIAMETER_SELECTED:    // -2
    str+= _T("Left click to move this diameter") + add_options;
    break;
  case m_dlg->NOTHING_SELECTED:   // -1
    if(m_dlg->m_over_root) str += _T("Left click to move root");
    else str += _T("Ready to modify root");
    
    str += add_options;
    break;
  default:
    str += _T("Left click to move this point") + add_options;
  }
  return str;
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_MOVINGRPOINT

void ModeHandlerMODE_MOVINGRPOINT::OnMouseMove(UINT nFlags, CPoint point) 
{
  SetCursor(m_dlg->m_cur_movepoint);
  RPoint tempdiam;
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  const RPoint oldpoint = *( inst->GetPoint(m_dlg->m_selected_index) );
  RPoint pt = m_dlg->m_converter.ClientToImage(CPoint(point.x,point.y),root->GetPrimaryWindow());
  
  // move point to a new location. call the Undo version of this command.
  m_dlg->m_controller.C_MovePoint( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.x, pt.y );
  
  m_dlg->m_controller.C_ChangeWindowOfPoint( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.window );
  
  // check if diameter is on relevant root segment, if it isn't, do nothing.
  if ( (m_dlg->section_start_point_index == m_dlg->m_selected_index) || (m_dlg->section_start_point_index == (m_dlg->m_selected_index-1) ) )
  {
    //get other point of segment
    const RPoint* otherpoint; 
    //get new diameter position if neccesary
    if ( m_dlg->section_start_point_index == m_dlg->m_selected_index )
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index + 1);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * m_dlg->diameter_position_ratio;
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * m_dlg->diameter_position_ratio;
    }
    else
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * (1 - m_dlg->diameter_position_ratio);
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * (1 - m_dlg->diameter_position_ratio);
    }
    tempdiam.window = diam->r_point.window;
    m_dlg->m_controller.C_MoveDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, tempdiam);
  }
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_MOVINGRPOINT::OnLButtonDown(UINT nFlags, CPoint point) 
{
  SetCursor(m_dlg->m_cur_movepoint);
  RPoint tempdiam;
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  const RPoint oldpoint = *( inst->GetPoint(m_dlg->m_selected_index) );
  RPoint pt = m_dlg->m_converter.ClientToImage(CPoint(point.x,point.y),root->GetPrimaryWindow());
  
  // move point to a new location. call the Undo version of this command.
  m_dlg->m_controller.C_MovePoint_U( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.x, pt.y, &(m_dlg->m_previous_root_for_undo));
  
  m_dlg->m_controller.C_ChangeWindowOfPoint( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.window );
  
  // check if diameter is on relevant root segment, if it isn't, do nothing.
  if ( (m_dlg->section_start_point_index == m_dlg->m_selected_index) || (m_dlg->section_start_point_index == (m_dlg->m_selected_index-1) ) )
  {
    //get other point of segment
    const RPoint* otherpoint; 
    //get new diameter position if neccesary
    if ( m_dlg->section_start_point_index == m_dlg->m_selected_index )
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index + 1);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * m_dlg->diameter_position_ratio;
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * m_dlg->diameter_position_ratio;
    }
    else
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * (1 - m_dlg->diameter_position_ratio);
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * (1 - m_dlg->diameter_position_ratio);
    }
    tempdiam.window = diam->r_point.window;
    m_dlg->m_controller.C_MoveDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, tempdiam);
  }
  
  m_dlg->SetMode(MODE_ROOTSELECTED);          
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_MOVINGRPOINT::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_MOVINGRPOINT::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_MOVINGRPOINT::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_MOVINGRPOINT::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_MOVINGRPOINT::GetStatusText() 
{
  return "Moving point, left click to finish"; 
}



////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_ADDPOINT

// created a new mode for add point. earlier, OnAddPoint from RootflyDlg set the
// mode to MOVINGRPOINT. this meant that the Undo functionality was unable to
// distinguish between a point being moved versus a new point being created and moved.
// thus, MOVINGRPOINT and ADDPOINT are almost the same. the exception is that
// the Undo function for MovePoint is called in MOVINGRPOINT, whereas the non-Undo
// version is called in ADDPOINT. - (Ninad Pradhan 04/25/08)
void ModeHandlerMODE_ADDPOINT::OnMouseMove(UINT nFlags, CPoint point) 
{
  SetCursor(m_dlg->m_cur_sizeall);
  RPoint tempdiam;
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  const RPoint oldpoint = *( inst->GetPoint(m_dlg->m_selected_index) );
  RPoint pt = m_dlg->m_converter.ClientToImage(CPoint(point.x,point.y),root->GetPrimaryWindow());
  
  // call the non-Undo version of MovePoint here.
  m_dlg->m_controller.C_MovePoint( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.x, pt.y );
  
  m_dlg->m_controller.C_ChangeWindowOfPoint( m_dlg->m_selected_index, m_dlg->m_selected_root_id, m_dlg->m_point_session, pt.window );
  
  // check if diameter is on relevant root segment, if it isn't, do nothing.
  if ( (m_dlg->section_start_point_index == m_dlg->m_selected_index) || (m_dlg->section_start_point_index == (m_dlg->m_selected_index-1) ) )
  {
    //get other point of segment
    const RPoint* otherpoint; 
    //get new diameter position if neccesary
    if ( m_dlg->section_start_point_index == m_dlg->m_selected_index )
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index + 1);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * m_dlg->diameter_position_ratio;
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * m_dlg->diameter_position_ratio;
    }
    else
    {
      otherpoint = inst->GetPoint(m_dlg->section_start_point_index);
      tempdiam.x = (double)pt.x - ( (double)pt.x - (double)otherpoint->x ) * (1 - m_dlg->diameter_position_ratio);
      tempdiam.y = (double)pt.y - ( (double)pt.y - (double)otherpoint->y ) * (1 - m_dlg->diameter_position_ratio);
    }
    tempdiam.window = diam->r_point.window;
    m_dlg->m_controller.C_MoveDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, tempdiam);
  }
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_ADDPOINT::OnLButtonDown(UINT nFlags, CPoint point) 
{
  m_dlg->SetMode(MODE_ROOTSELECTED);          
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_ADDPOINT::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_ADDPOINT::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_ADDPOINT::OnRButtonDown(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_ADDPOINT::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_ADDPOINT::GetStatusText() 
{
  return "Moving point, left click to finish"; 
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_EXTENDINGENDPOINT

void ModeHandlerMODE_EXTENDINGENDPOINT::OnMouseMove(UINT nFlags, CPoint point) 
{
  CPoint head = m_dlg->m_converter.ImageToClient(*(m_dlg->m_current_root_being_drawn.GetPoint(0)));
  const vector<RPoint> *points = m_dlg->m_current_root_being_drawn.GetAllPoints();    
  CPoint tail = m_dlg->m_converter.ImageToClient(*(m_dlg->m_current_root_being_drawn.GetPoint(points->size()-1)));
  
  double delta_x, delta_y;
  delta_x = point.x - head.x;
  delta_y = point.y - head.y;
  double distance_to_head = delta_x * delta_x + delta_y * delta_y;
  delta_x = point.x - tail.x;
  delta_y = point.y - tail.y;
  double distance_to_tail = delta_x * delta_x + delta_y * delta_y;
  // if distance_to_tail == distance_to_head, both would be false
  m_dlg->m_extending_from_tail = (distance_to_tail < distance_to_head) ? true : false;
  m_dlg->m_extending_from_head = !(m_dlg->m_extending_from_tail);
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_EXTENDINGENDPOINT::OnLButtonDown(UINT nFlags, CPoint point) 
{
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName() );
  assert(tube); if (!tube) return;
  const Root *tempr = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(tempr);  if (!tempr)  return;
  const RootInstance *inst = tempr->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  RPoint pt = m_dlg->m_converter.ClientToImage(CPoint(point.x,point.y),tempr->GetPrimaryWindow());
  
  RootInstance old_ri;
  
  const vector<Root> *roots = tube->GetAllRoots();
  assert(roots); if (!roots) return;
  
  for (int i = 0; i < roots->size(); i++)
  {
    const Root& root = roots->at(i);
    int id = root.GetID();
    if (id == m_dlg->m_selected_root_id)
    {
      const RootInstance *inst = root.GetRootInstance(m_dlg->m_point_session);  
      if (!inst) return;
      old_ri = *inst;
      RPoint *rpt = m_dlg->m_controller.C_CreatePoint(id, m_dlg->m_point_session);
      //RPoint *rpt = m_dlg->m_controller.C_CreatePoint(id);
      if (m_dlg->m_extending_from_tail)
      {
        *rpt = pt;
      }
      if (m_dlg->m_extending_from_head)
      {
        const std::vector<RPoint>* polyline = inst->GetAllPoints();
        assert(polyline);  if (!polyline)  return;
        for (int j = polyline->size()-1 ; j >= 0 ; j--)
        {
          RPoint temp_point = (j>0) ? polyline->at(j-1) : pt;
          m_dlg->m_controller.C_MovePoint(j, id, m_dlg->m_point_session, temp_point.x, temp_point.y);
          m_dlg->m_controller.C_ChangeWindowOfPoint(j, id, m_dlg->m_point_session, temp_point.window);
        }
      }
      break;
    }
  } 
  
  // store on the undo vector
  m_dlg->m_controller.C_ExtendEndpoint(old_ri, m_dlg->m_selected_root_id, m_dlg->m_point_session);
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_EXTENDINGENDPOINT::OnLButtonUp(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_EXTENDINGENDPOINT::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
  m_dlg->SetMode(MODE_IMAGEDISPLAYED);
  m_dlg->m_extending_from_tail = false;
  m_dlg->m_extending_from_head = false;
  m_dlg->m_current_root_being_drawn.ClearAll();
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_EXTENDINGENDPOINT::OnRButtonDown(UINT nFlags, CPoint point) 
{
  m_dlg->SetMode(MODE_ROOTSELECTED);
  m_dlg->m_extending_from_tail = false;
  m_dlg->m_extending_from_head = false;
  m_dlg->m_current_root_being_drawn.ClearAll();
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_EXTENDINGENDPOINT::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_EXTENDINGENDPOINT::GetStatusText() 
{ 
  return "Left click to extend endpoint, double-click to finish";
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_IMAGEDISPLAYED

void ModeHandlerMODE_IMAGEDISPLAYED::OnMouseMove(UINT nFlags, CPoint point) 
{
  int point_session = m_dlg->m_point_session;
  int   window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  RPoint pt = m_dlg->m_converter.ClientToImage( CPoint(point.x,point.y),window);
  int tempRootID;	
  RDiameter nearest_point;
  m_dlg->FindNearestPoint( pt.x, pt.y, &nearest_point);      
  int diameter_selected = m_dlg->FindNearestDiameter( pt.x, pt.y );
  if ( nearest_point.radius < 0 && diameter_selected < 0)
  { // mouse is not over on anything
    m_dlg->m_selected_root_id = -1;
    m_dlg->m_selected_root_session = -1;
    if ( m_dlg->m_over_root )
    {
      m_dlg->m_over_root = false; 
      m_dlg->ReDraw(1, 0);          
    }
  }
  else 
  { 
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName( m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName() );
    if (!tube)  return;
    
    if (nearest_point.radius >= 0)
    { // mouse is over polyline
      tempRootID = nearest_point.r_point.window;
      const Root *root = tube->GetRoot(tempRootID);
      if (!root)  return;
      const RootInstance *inst = root->GetRootInstance(point_session);
      if (!inst)  return;
      if ((inst->IsAlive()) || (m_dlg->m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED))
      {
        m_dlg->m_selected_root_id = nearest_point.r_point.window;
        m_dlg->m_selected_root_session = point_session;
      }
      SetCursor(m_dlg->m_cur_sizeall); 
    }
    else
    { // mouse is over diameter
      tempRootID = diameter_selected;
      const Root *root = tube->GetRoot(tempRootID);
      if (!root)  return;
      const RootInstance *inst2 = root->GetRootInstance(point_session);
      if (!inst2)  return;
      if ((inst2->IsAlive()) || (m_dlg->m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED)) 
      {
        m_dlg->m_selected_root_id =  diameter_selected;   
        m_dlg->m_selected_root_session = point_session;
      }
    }
    
    if ( !m_dlg->m_over_root )
    {
      m_dlg->m_over_root = true;
      m_dlg->ReDraw(1, 0);
    }
    else
    {
      m_dlg->ReDraw(1, 0);    
    }
  }
}

/* If mouse is not hovering over a root, user's click-and-hold of mouse button should move the image around,
* handled by MODE_PANNINGIMAGE.
*/
void ModeHandlerMODE_IMAGEDISPLAYED::OnLButtonDown(UINT nFlags, CPoint point) 
{
  if (m_dlg->m_over_root) m_dlg->SetMode(MODE_ROOTSELECTED);            
  else if (m_dlg->m_zoomin != 1) 
  {
    SetCursor(m_dlg->m_cur_closedhand);
    m_dlg->SetMode(MODE_PANNINGIMAGE);
    m_dlg->m_last_mouse_move_point = point;
  }
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_IMAGEDISPLAYED::OnLButtonUp(UINT nFlags, CPoint point) 
{
  
}

void ModeHandlerMODE_IMAGEDISPLAYED::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
}

void ModeHandlerMODE_IMAGEDISPLAYED::OnRButtonDown(UINT nFlags, CPoint point) 
{
  MenuSetup(point);
}

void ModeHandlerMODE_IMAGEDISPLAYED::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

CString ModeHandlerMODE_IMAGEDISPLAYED::GetStatusText() 
{ 
  const ApplicationData &app = m_dlg->m_rfy_data->GetApplicationData();
  const Experiment &exp = m_dlg->m_rfy_data->GetExperiment();
  const Tube *tube = exp.GetTubeByName(app.GetCurrentTubeName());
  if(!tube) return "";
  assert(tube);
  int curr_window = app.GetCurrentWindow();
  int curr_session = app.GetCurrentSession();
  
  if (tube->GetImageAnnotated(curr_window, curr_session) )
  {
    return "Ready to annotate, right click for additional options";
  }
  else
  {
    return "Click the Annotate button or press Ctrl+A to begin annotating...";
  }
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_SIZINGDIAMETER

void ModeHandlerMODE_SIZINGDIAMETER::OnMouseMove(UINT nFlags, CPoint point)
{
  SetCursor(m_dlg->m_cur_sizens);    
  ResizeCurrentDiameterUsingScalingFactor(point, m_dlg);
//  { // same code as ModeHandlerMODE_LOCATINGDIAMETER::OnLButtonDown
//    vector<int> window_list;
//    m_dlg->m_current_root_being_drawn.GetWindowNumbers(&window_list);
//    RPoint pt = m_dlg->m_converter.ClientToImage(point);
//    pt = m_dlg->m_converter.ChangeWindow(pt ,window_list[0]);
//    RDiameter *rdm = m_dlg->m_current_root_being_drawn.GetDiameter(0);
//        
//    double factor = m_dlg->m_rfy_data->GetSettings().GetDiameterScalingFactor();
//    factor = 3;
//    
//    int initial_x = rdm->r_point.x;
//    int x_difference = floor(fabs(initial_x - pt.x) / factor);
//    int initial_y = rdm->r_point.y;
//    int y_difference = floor(fabs(initial_y - pt.y) / factor);
//    
//    if (initial_x > pt.x)
//      pt.x = initial_x - x_difference;
//    else pt.x = initial_x + x_difference;
//    if (initial_y > pt.y)
//      pt.y = initial_y - y_difference;
//    else pt.y = initial_y + y_difference;
//    
//    rdm->radius = sqrt( pow(( (float)pt.x - initial_x),2) + pow(( (float)pt.y - initial_y),2) );
//  }
  m_dlg->ReDraw(1, 0); 
}

void ModeHandlerMODE_SIZINGDIAMETER::OnLButtonDown(UINT nFlags, CPoint point2)
{
  // User has finished drawing new root, so call CreateRoot function to create root with this root instance
  const std::vector<RPoint> *line; 
  line = m_dlg->m_current_root_being_drawn.GetAllPoints();    
  if (line->size() == 0)  return;
  m_dlg->m_controller.C_CreateRoot( line->at(0).window, m_dlg->m_current_root_being_drawn, m_dlg->GetConditionDefault() );
  
  m_dlg->m_selected_root_id = -1;
  m_dlg->m_selected_root_session = -1;
  m_dlg->SetMode(MODE_IMAGEDISPLAYED);
  m_dlg->m_current_root_being_drawn.ClearAll();
  m_dlg->ReDraw(1, 0);
}

void ModeHandlerMODE_SIZINGDIAMETER::OnLButtonUp(UINT nFlags, CPoint point2)
{
}

void ModeHandlerMODE_SIZINGDIAMETER::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_SIZINGDIAMETER::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_SIZINGDIAMETER::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_SIZINGDIAMETER::GetStatusText() 
{ 
  return "Left click to set diameter size";
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_NOIMAGEDISPLAYED

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnMouseMove(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnLButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnLButtonUp(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_NOIMAGEDISPLAYED::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_NOIMAGEDISPLAYED::GetStatusText()
{
  return "No Image Displayed";
}


////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_RESIZINGDIAMETER

void ModeHandlerMODE_RESIZINGDIAMETER::OnMouseMove(UINT nFlags, CPoint point2)
{
  ResizeDiameterUsingScalingFactor(point2, m_dlg);
  /* good code
//<<<<<<< ModeHandler.cpp
    SetCursor(m_dlg->m_cur_sizens);    
    const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
    assert(root);  if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_dlg->m_rfy_data->GetApplicationData().GetCurrentSession());
    assert(inst);  if (!inst)  return;
    const RDiameter *diam = inst->GetDiameter(0);
    assert(diam);  if (!diam)  return;
    RPoint point = m_dlg->m_converter.ClientToImage( CPoint(point2.x,point2.y),root->GetPrimaryWindow());
    {        
		double factor = m_dlg->m_rfy_data->GetSettings().GetDiameterScalingFactor();
		factor = 3;
        
		int delta_x, delta_y;
		delta_x = floor(fabs(diam->r_point.x - point.x)/factor);
        delta_y = floor(fabs(diam->r_point.y - point.y)/factor);

		if (diam->r_point.x > point.x)
			point.x = diam->r_point.x - delta_x;
		else
			point.x = diam->r_point.x + delta_x;

		if (diam->r_point.y > point.y)
			point.y = diam->r_point.y - delta_y;
		else
			point.y = diam->r_point.y + delta_y;

        double new_radius = sqrt( pow(( (float)point.x - diam->r_point.x),2) + pow(( (float)point.y - diam->r_point.y),2) );
        //m_dlg->m_controller.C_ResizeDiameter(0, m_dlg->m_selected_root_id, new_radius);
        m_dlg->m_controller.C_ResizeDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, new_radius);      
    }
    */

    m_dlg->ReDraw(1,0);
/*=======
  SetCursor(m_dlg->m_cur_sizens);    
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_dlg->m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  RPoint point = m_dlg->m_converter.ClientToImage( CPoint(point2.x,point2.y),root->GetPrimaryWindow());
  {        
    int delta_x, delta_y;        
    delta_x = diam->r_point.x - point.x;
    delta_y = diam->r_point.y - point.y;
    
    double factor = m_dlg->m_rfy_data->GetSettings().GetDiameterScalingFactor();
    //		double factor = m_dlg->m_diameter_scaling_factor;
    factor = 3;
    
    double new_radius = sqrt(pow((float)delta_x,2)+pow((float)delta_y,2)) / factor;
    m_dlg->m_controller.C_ResizeDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, new_radius);      
  }
  m_dlg->ReDraw(1,0);
>>>>>>> 1.116
*/
}

void ModeHandlerMODE_RESIZINGDIAMETER::OnLButtonDown(UINT nFlags, CPoint point2)
{
  ResizeDiameterUsingScalingFactor(point2, m_dlg);
  /* good code
//<<<<<<< ModeHandler.cpp
SetCursor(m_dlg->m_cur_sizens);    
const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
assert(tube);  if (!tube)  return;
const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
assert(root);  if (!root)  return;
const RootInstance *inst = root->GetRootInstance(m_dlg->m_rfy_data->GetApplicationData().GetCurrentSession());
assert(inst);  if (!inst)  return;
const RDiameter *diam = inst->GetDiameter(0);
assert(diam);  if (!diam)  return;
RPoint point = m_dlg->m_converter.ClientToImage( CPoint(point2.x,point2.y),root->GetPrimaryWindow());
{	 
  double factor = m_dlg->m_rfy_data->GetSettings().GetDiameterScalingFactor();
	factor = 3;
	
	int delta_x, delta_y;
	delta_x = floor(fabs(diam->r_point.x - point.x)/factor);
	delta_y = floor(fabs(diam->r_point.y - point.y)/factor);
	
	if (diam->r_point.x > point.x)
		point.x = diam->r_point.x - delta_x;
	else
		point.x = diam->r_point.x + delta_x;
	
	if (diam->r_point.y > point.y)
		point.y = diam->r_point.y - delta_y;
	else
		point.y = diam->r_point.y + delta_y;
	
	double new_radius = sqrt( pow(( (float)point.x - diam->r_point.x),2) + pow(( (float)point.y - diam->r_point.y),2) );
	m_dlg->m_controller.C_ResizeDiameter(0, m_dlg->m_selected_root_id, m_dlg->m_point_session, new_radius);      
	//m_dlg->m_controller.C_ResizeDiameter_U(0, m_dlg->m_selected_root_id, m_dlg->m_previous_radius_for_undo, new_radius);	  
}
*/
m_dlg->SetMode(MODE_ROOTSELECTED);
m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_RESIZINGDIAMETER::OnLButtonUp(UINT nFlags, CPoint point2)
{
}

void ModeHandlerMODE_RESIZINGDIAMETER::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_RESIZINGDIAMETER::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_RESIZINGDIAMETER::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_RESIZINGDIAMETER::GetStatusText()
{
  return "Resizing diameter, left click to finish";
}

////////////////////////////////////////////////////////////////////////
// ModeHandlerMODE_PANNINGIMAGE

void ModeHandlerMODE_PANNINGIMAGE::OnMouseMove(UINT nFlags, CPoint point)
{
  SetCursor(m_dlg->m_cur_closedhand);
  
  CPoint last = m_dlg->m_last_mouse_move_point;
  m_dlg->m_shiftx -= (int)(point.x - last.x);
  m_dlg->m_shifty -= (int)(point.y - last.y);
  m_dlg->m_last_mouse_move_point = point;
  
  m_dlg->SetZoomWindows();
  m_dlg->ReDraw(1,0);
}

void ModeHandlerMODE_PANNINGIMAGE::OnLButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_PANNINGIMAGE::OnLButtonUp(UINT nFlags, CPoint point)
{
  m_dlg->SetMode(MODE_IMAGEDISPLAYED);
}

void ModeHandlerMODE_PANNINGIMAGE::OnLButtonDblClk(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_PANNINGIMAGE::OnRButtonDown(UINT nFlags, CPoint point)
{
}

void ModeHandlerMODE_PANNINGIMAGE::OnRButtonUp(UINT nFlags, CPoint point)
{
}

CString ModeHandlerMODE_PANNINGIMAGE::GetStatusText()
{
  return "Panning Image, release mouse button to stop panning.";
}


// this function sets up the context menu that appears when the user right clicks.  Different
// options in that menu are disabled or enabled depending on the current mode and what was 
// right-clicked on.  There are 2 CMenu objects declared; this was the result of an early 
// implementation and this function can probably be modified to use only one.  To modify the menu
// options, click on the resource tab in the Workspace pane and open the IDR_MENU_CONTEXT menu
// INPUTS:
//   point:  mouse location
//   clicked_point_of_selected_root:  whether user clicked on a point of the selected root
void ModeHandler::MenuSetup(CPoint point)
{
  int point_session = m_dlg->m_point_session;
  const Tube *tube = m_dlg->m_rfy_data->GetExperiment().GetTubeByName(m_dlg->m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);
  if (!tube)  return;
  
  
  int curr_window = m_dlg->m_rfy_data->GetApplicationData().GetCurrentWindow();
  int curr_session = point_session;
  
  CMenu contextMenu;
  CMenu *rootPopup;
  
  contextMenu.LoadMenu(IDR_MENU_CONTEXT);
  rootPopup = contextMenu.GetSubMenu(0);
  
  // disable all menu items:  The purpose of this is so they can be re-enabled based on Menu Command ID
  // instead of Menu Command Position.  Presently, both menus options with sub menus (State and Color)
  // should be enabled and disabled together, so this allows the enabling and disabling to be done
  // without needing their aboslute position (absolute positions will change if menu options are added or 
  // substracted).  There are more sophisticated ways to disable and enable entire
  // submenus individually without resorting to the MF_BYPOSITION flag if someone would like to try to
  // implement that
  for (int i=0; i<rootPopup->GetMenuItemCount(); i++)
  {
    rootPopup->EnableMenuItem(i, MF_BYPOSITION | MF_GRAYED);
  }
  
  switch(m_dlg->GetMode())
  {
  case MODE_PANNINGIMAGE:
  case MODE_IMAGEDISPLAYED:
    {
      if (tube->GetImageAnnotated(curr_window, curr_session))   // if the image is already annotated
      {
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_MOVEALLROOTS, MF_BYCOMMAND | MF_ENABLED);
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_COPY, MF_BYCOMMAND | MF_ENABLED);
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_AUTODETECTROOTS, MF_BYCOMMAND | MF_ENABLED);
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_NEWROOT, MF_BYCOMMAND | MF_ENABLED);
      }
      else  // image is not annotated
      {
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_ANNOTATE, MF_BYCOMMAND | MF_ENABLED);
      }
      
      break;
    }
  case MODE_MOVINGRPOINT:
  case MODE_MOVINGSINGLEROOT:
  case MODE_ROOTSELECTED:
    {
      const Root *root = tube->GetRoot(m_dlg->m_selected_root_id);
      assert(root);
      if (!root)  return;
      const RootInstance *inst = root->GetRootInstance(curr_session);
      assert(inst);
      if (!inst)  return;
      
      for (int i=0; i<rootPopup->GetMenuItemCount(); i++)
      {
        if (rootPopup->GetMenuItemID(i) == -1)
        {
          rootPopup->EnableMenuItem(i, MF_BYPOSITION | MF_ENABLED);
        }
      }
      
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_MOVEALLROOTS, MF_BYCOMMAND | MF_ENABLED);
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_COPY, MF_BYCOMMAND | MF_ENABLED);
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_AUTODETECTROOTS, MF_BYCOMMAND | MF_ENABLED);
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_NEWROOT, MF_BYCOMMAND | MF_ENABLED);
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_DELETEROOT, MF_BYCOMMAND | MF_ENABLED);
      rootPopup->EnableMenuItem(ID_MENUCONTEXT_EXTENDENDPOINT, MF_BYCOMMAND | MF_ENABLED);
      
      if (m_dlg->m_selected_index > -1) // the point of a root was right-clicked
      {
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_DELETEPOINT, MF_BYCOMMAND | MF_ENABLED);
      }
      else    // the line segment of a root was clicked
      {
        m_dlg->temporary_point_to_add = m_dlg->m_converter.ClientToImage(point);
        m_dlg->FindNearestPointOnRootSeg(m_dlg->temporary_point_to_add, *inst, &(m_dlg->section_start_point_index));  //Get segment to add point on
        
        rootPopup->EnableMenuItem(ID_MENUCONTEXT_ADDPOINT, MF_BYCOMMAND | MF_ENABLED);
      }
      // checks the appropriate Live or Dead root state in the menu
      if (inst->IsAlive()) rootPopup->CheckMenuItem(ID_MENUCONTEXT_STATELIVE, MF_BYCOMMAND | MF_CHECKED);
      else                 rootPopup->CheckMenuItem(ID_MENUCONTEXT_STATEDEAD, MF_BYCOMMAND | MF_CHECKED);
      
      // checks the appropriate root Color in the menu
      int color = inst->GetColor();
      
      switch(color)
      {
      case inst->COLOR_WHITE:
        rootPopup->CheckMenuItem(ID_MENUCONTEXT_COLORWHITE, MF_BYCOMMAND | MF_CHECKED);
        break;
      case inst->COLOR_LIGHTBROWN:
        rootPopup->CheckMenuItem(ID_MENUCONTEXT_COLORLIGHTBROWN, MF_BYCOMMAND | MF_CHECKED);
        break;
      case inst->COLOR_DARKBROWN:
        rootPopup->CheckMenuItem(ID_MENUCONTEXT_COLORDARKBROWN, MF_BYCOMMAND | MF_CHECKED);
        break;
      case inst->COLOR_BLACK:
        rootPopup->CheckMenuItem(ID_MENUCONTEXT_COLORBLACK, MF_BYCOMMAND | MF_CHECKED);
        break;
      default:  // this should not be reached, but check the White color if it is
        rootPopup->CheckMenuItem(ID_MENUCONTEXT_COLORWHITE, MF_BYCOMMAND | MF_CHECKED);
        break;
      }
      
      break;
    }
  default:
    break;
  }
  CPoint pt = point;
  m_dlg->ClientToScreen(&pt);
  rootPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_dlg);
//  SetCursorPos(pt.x, pt.y);  // This makes the behavior for 'Add point' more intuitive, but some have complained that it has negative consequences elsewhere -- STB 5/4/10
}
